#include "aboutdialog.h"

#include <QtCore/QFile>
#include <QtCore/QTextStream>
#include <QtCore/QLocale>
#include <QtGui/QTabWidget>
#include <QtGui/QWidget>
#include <QtGui/QLayout>
#include <QtGui/QTextEdit>
#include <QtGui/QTextBrowser>
#include <QtGui/QDialogButtonBox>
#include <QtGui/QLabel>
#include <QtGui/QSpacerItem>

AboutDialog::AboutDialog(){
    resize(600,400);
    //setWindowTitle(QString::fromUtf8("Névjegy"));
    setWindowTitle(tr("About"));

    QString aboutPath = ":/texts/about_en.txt";
    QString rulesPath = ":/texts/rules_en.txt";

    if(QLocale::system().language() == QLocale::Hungarian){
        aboutPath = ":/texts/about_hu.txt";
        rulesPath = ":/texts/rules_hu.txt";
    }

    QTextEdit *rulesTextEdit = new QTextEdit();
    rulesTextEdit->setPlainText(getStringFromResource(rulesPath));
    rulesTextEdit->setReadOnly(true);

    QTextEdit *licenseTextEdit = new QTextEdit();
    licenseTextEdit->setPlainText(getStringFromResource(QString(":/Copying.txt")));
    licenseTextEdit->setReadOnly(true);

    tabWidget = new QTabWidget();
    //tabWidget->addTab(aboutTextEdit,QString::fromUtf8("Névjegy"));
    //tabWidget->addTab(aboutTextEdit,tr("About"));
    tabWidget->addTab(getAboutWidget(aboutPath),tr("About"));
    //tabWidget->addTab(rulesTextEdit,QString::fromUtf8("Játékszabály"));
    tabWidget->addTab(rulesTextEdit,tr("Game rules"));
    //tabWidget->addTab(licenseTextEdit,QString::fromUtf8("Licenc"));
    tabWidget->addTab(licenseTextEdit,tr("License"));

    tabWidget->setTabEnabled(1,false);

    QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok);
    connect(buttonBox,SIGNAL(accepted()),this,SLOT(close()));

    QVBoxLayout *mainLayout = new QVBoxLayout();
    mainLayout->addWidget(tabWidget);
    mainLayout->addWidget(buttonBox);

    setLayout(mainLayout);
}

/*AboutDialog::AboutDialog(){



}*/

QWidget *AboutDialog::getAboutWidget(QString path){
    QWidget *aboutWidget = new QWidget();

    QLabel *logo = new QLabel();
    logo->setPixmap(QPixmap(":/pics/icon.png"));

    QSpacerItem *spacer = new QSpacerItem(20,111,QSizePolicy::Minimum,QSizePolicy::Expanding);
    spacer->setAlignment(Qt::AlignTop);

    QVBoxLayout *logoLayout = new QVBoxLayout();
    //logoLayout->setSpacing(6);
    //logoLayout->addSpacerItem(spacer);
    logoLayout->addWidget(logo);
    //logoLayout->addLayout(spacer);
    //logoLayout->addSpacerItem(spacer);
    logoLayout->addSpacerItem(spacer);


    QTextBrowser *aboutTextBrowser = new QTextBrowser();
    aboutTextBrowser->setPlainText(getStringFromResource(path));
    aboutTextBrowser->setReadOnly(true);

    QHBoxLayout *aboutLayout = new QHBoxLayout();
    aboutLayout->addLayout(logoLayout);
    aboutLayout->addWidget(aboutTextBrowser);

    aboutWidget->setLayout(aboutLayout);
    return aboutWidget;
}

QString AboutDialog::getStringFromResource(QString path){
    QString return_string;
    QFile file(path);
    if (file.open(QIODevice::ReadOnly)){
        QTextStream ts(&file);
        ts.setCodec("UTF-8");
        return_string = ts.readAll();
        file.close();
    }
    return return_string;
}

AboutDialog::~AboutDialog(){
    delete tabWidget;
}
