/*
 * This class set the cards position and the points, etc
 *
 */
#include <QtCore/QTimer>
#include <QtCore/QPoint>
#include <QtCore/QSize>
#include <QtGui/QImage>

#include <iostream>

#include "desktopcore.h"
#include "bot.h"

DesktopCore::DesktopCore(MainWindow *m_mainWindow){
    mainWindow = m_mainWindow;
    bot = NULL;
    cardUtil = new CardUtil();
    enemyPlayer = new Player();
    humanPlayer = new HumanPlayer();
    //The hungarian number of cards is 32
    setCardSum();
    inGame = false;
    //inTimer = false;

    //speed = 1000;

    botTimer = new QTimer();
    botTimer->setSingleShot(true);
    botTimer->setInterval(1000);
    connect(botTimer,SIGNAL(timeout()),this,SLOT(botInAction()));

    dcicTimer = new QTimer();
    dcicTimer->setSingleShot(true);
    dcicTimer->setInterval(1000);
    connect(dcicTimer,SIGNAL(timeout()),this,SLOT(deleteCardsInCentral()));


    //Card isn't in the central
    cardUtil->deleteCardsInCentral(cardsInCentral);

    cardUtil->deleteDeckOfCards(sortOfDeckOfCards);

    isPlayerStartNextTurn = false;
    isPlayerWonTheLastTurn = false;

    backCard = QString(":/pics/cards/back1.png");
}

void DesktopCore::setBot(){
    bot = new Bot();
}

//set the size of main window
void DesktopCore::setDesktopSize(QSize size){
    windowSize = size;
}

int* DesktopCore::getSortOfDeckOfCards(){
    return sortOfDeckOfCards;
}

void DesktopCore::stopTimers(){
    if(botTimer->isActive())
        botTimer->stop();
    if(dcicTimer->isActive())
        dcicTimer->stop();
}

void DesktopCore::setSpeed(int mspeed){
    botTimer->setInterval(mspeed);
    dcicTimer->setInterval(mspeed);
}

int DesktopCore::getSpeed(){
    return botTimer->interval();
}

//add the deck of cards to the main window
void DesktopCore::addDeckOfCards(){
    QPoint tmpPoint;
    tmpPoint.setX(20);
    tmpPoint.setY(int((windowSize.height()-cardUtil->getCardSize().height())/2)-20);

    if(cardSum != -1){
        //QString textStr = (QString::number(cardSum+1))+QString::fromUtf8(" kártya");
        QString textStr = (QString::number(cardSum+1))+tr(" cards");
        mainWindow->desktopView->drawText(QPoint(tmpPoint.x(),tmpPoint.y()-10),textStr);
    }

    for (int i=0;i<cardSum;i++){
        if(i%6==0){
            tmpPoint.setX(tmpPoint.x()+1);
            tmpPoint.setY(tmpPoint.y()-1);
        }
        //mainWindow->desktopView->drawCard(tmpPoint,QImage(QString(":/pics/cards/back1.png")),false);
        //mainWindow->desktopView->drawCard(tmpPoint,QImage(backCard),false);
        mainWindow->desktopView->drawCard(tmpPoint,0,false);
    }
}

void DesktopCore::setCardsPosition(){
    int tmp;

    tmp = int(windowSize.rwidth()/2)-5-cardUtil->getCardSize().width()-10-cardUtil->getCardSize().width();
    humanPlayer->cardsPosition[0].setX(tmp);
    enemyPlayer->cardsPosition[0].setX(tmp);

    tmp = int(windowSize.rwidth()/2)-5-cardUtil->getCardSize().width();
    humanPlayer->cardsPosition[1].setX(tmp);
    enemyPlayer->cardsPosition[1].setX(tmp);

    tmp = int(windowSize.rwidth()/2)+5;
    humanPlayer->cardsPosition[2].setX(tmp);
    enemyPlayer->cardsPosition[2].setX(tmp);

    tmp=int(windowSize.rwidth()/2)+5+cardUtil->getCardSize().width()+10;
    humanPlayer->cardsPosition[3].setX(tmp);
    enemyPlayer->cardsPosition[3].setX(tmp);

    tmp=windowSize.rheight()-50-cardUtil->getCardSize().height();
    humanPlayer->cardsPosition[0].setY(tmp);
    humanPlayer->cardsPosition[1].setY(tmp);
    humanPlayer->cardsPosition[2].setY(tmp);
    humanPlayer->cardsPosition[3].setY(tmp);

    tmp=20;

    enemyPlayer->cardsPosition[0].setY(tmp);
    enemyPlayer->cardsPosition[1].setY(tmp);
    enemyPlayer->cardsPosition[2].setY(tmp);
    enemyPlayer->cardsPosition[3].setY(tmp);

    //The central cards position
    if(cardUtil->howManyCardsInCentral(cardsInCentral) != -1)
        setCardsInCentralPosition();
}

void DesktopCore::setCardsInCentralPosition(){
    cardsInCentralPosition[0].setX((int)((windowSize.width()-cardUtil->getCardSize().width())/2-cardUtil->howManyCardsInCentral(cardsInCentral)*(cardUtil->getCardSize().width()/4)));
    cardsInCentralPosition[0].setY((int)windowSize.height()/2-cardUtil->getCardSize().height()/2-20);

    for(int i=1;i<cardUtil->howManyCardsInCentral(cardsInCentral)+1;i++){
        cardsInCentralPosition[i].setX(cardsInCentralPosition[i-1].x()+(int)cardUtil->getCardSize().width()/2);
        cardsInCentralPosition[i].setY(cardsInCentralPosition[i-1].y());
    }
}

void DesktopCore::deleteCardsInCentralFromTable(){
    for(int i=0;i<8;i++){
        if(cardsInCentral[i] != -1){
            //Ezt meg at kell dolgozni, mert utolso kartyaig fel kartyak vennek ez meg egeszet torol.
            deleteRect(cardsInCentralPosition[i],cardUtil->getCardSize());
        }
    }
}

void DesktopCore::addCardsInCentralToTable(){
    for(int i=0;i<8;i++){
        if(cardsInCentral[i] != -1){
            //mainWindow->desktopView->drawCard(cardsInCentralPosition[i],QImage(QString(":/pics/cards/"+QString::number(cardsInCentral[i])+".png")),false);
            mainWindow->desktopView->drawCard(cardsInCentralPosition[i],cardsInCentral[i],false);
        }
    }
}

void DesktopCore::startGame(int gameMode){
    humanPlayer->deletePoints();
    enemyPlayer->deletePoints();

    if(getCardSum() != 31)
        setCardSum();

    //tmp
    //cardSum = 7;
    ///tmp

    if(!bot)
        setBot();

    setInGame(true);

    setCardsPosition();
    humanPlayer->setPlayerCardsRect(cardUtil->getCardSize(),mainWindow->desktopView->menuBarSize);
    cardUtil->setDeckOfCards(sortOfDeckOfCards);
    addCardsToEnemyAndPlayer();

    //If random
    if(gameMode == 2){
        gameMode = qrand()%2;
    }
    mainWindow->setWhoStartTheLastRound(gameMode);
    if(gameMode == 0){
        humanPlayer->setTrueToAllEnabledCards();
        mainWindow->setStatusBarMessage(DesktopCore::PlayerStart);
    }else{
        //QTimer::singleShot(speed,this,SLOT(botInAction()));
        //timer->singleShot(speed,this,SLOT(botInAction()));
        botTimer->start();
        mainWindow->setStatusBarMessage(DesktopCore::EnemyStart);
    }
}

void DesktopCore::addCardsToEnemyAndPlayer(){
    if(cardSum >= 1){
        for(int i=0;i<4;i++){
            if(cardSum >= 1){
                for(int j=0;j<4;j++){
                    if(enemyPlayer->cards[j] == -1){
                        enemyPlayer->cards[j] = sortOfDeckOfCards[cardSum];
                        cardSum--;
                        if(inGame){
                            //mainWindow->desktopView->drawCard(enemyPlayer->cardsPosition[j] ,QImage(backCard),false);
                            mainWindow->desktopView->drawCard(enemyPlayer->cardsPosition[j] ,0,false);
                        }
                        break;
                    }
                }
                for(int j=0;j<4;j++){
                    if(humanPlayer->cards[j] == -1){
                        humanPlayer->cards[j] = sortOfDeckOfCards[cardSum];
                        cardSum--;

                        if(inGame){
                            mainWindow->desktopView->drawCard(humanPlayer->cardsPosition[j],humanPlayer->cards[j],false);
                        }
                        break;
                    }
                }
            }
        }
        // tmp
        //repaint cardsOfDeck
        deleteDeckOfCardsRect();
        addDeckOfCards();
        // /tmp
    }
}

void DesktopCore::deleteDeckOfCardsRect(){
    //std::cout << "Torol deckofcardsrect" << std::endl;
    deleteRect(QPoint(20,int((windowSize.height()-cardUtil->getCardSize().height())/2-40)),QSize(cardUtil->getCardSize().width()+6,cardUtil->getCardSize().height()+20));
}

void DesktopCore::addCardsToTable(){
    //enemy and player cards
    for(int i=0;i<4;i++){
        if(enemyPlayer->cards[i] != -1){
            //mainWindow->desktopView->drawCard(enemyPlayer->cardsPosition[i],QImage(backCard),false);
            mainWindow->desktopView->drawCard(enemyPlayer->cardsPosition[i],0,false);
            //mainWindow->desktopView->drawCard(enemyPlayer->cardsPosition[i],QImage(QString(":/pics/cards/"+QString::number(enemyPlayer->cards[i])+".png")),false);
        }
        if(humanPlayer->cards[i] != -1){
            //mainWindow->desktopView->drawCard(humanPlayer->cardsPosition[i],QImage(QString(":/pics/cards/"+QString::number(humanPlayer->cards[i])+".png")),false);
            mainWindow->desktopView->drawCard(humanPlayer->cardsPosition[i],humanPlayer->cards[i],false);
        }
    }
    //central cards
    for(int i=0;i<8;i++)
        if(cardsInCentral[i] != -1)
            //mainWindow->desktopView->drawCard(cardsInCentralPosition[i],QImage(QString(":/pics/cards/"+QString::number(cardsInCentral[i])+".png")),false);
            mainWindow->desktopView->drawCard(cardsInCentralPosition[i],cardsInCentral[i],false);
}

void DesktopCore::deleteRect(QPoint point,QSize size){
    mainWindow->desktopView->clearBox(QRect(point,size));
}

void DesktopCore::botInAction(){
    enemyClickToCard(bot->botInAction(enemyPlayer->cards,cardsInCentral));
    mainWindow->update();

    //end the QTimer
    //inTimer = false;
}

void DesktopCore::enemyClickToCard(int clickedCardNumber){
    mainWindow->setStatusBarMessage(0);
    deleteCardsInCentralFromTable();

    for(int i=0; i<8;i++){
        if(cardsInCentral[i] == -1){
            cardsInCentral[i] = enemyPlayer->cards[clickedCardNumber];
            break;
        }
    }
    setCardsInCentralPosition();
    addCardsInCentralToTable();

    enemyPlayer->cards[clickedCardNumber] = -1;

    deleteRect(enemyPlayer->cardsPosition[clickedCardNumber],cardUtil->getCardSize());

    whoWinTheRound(false,false);
}

void DesktopCore::playerClickToCard(int clickedCardNumber){
    mainWindow->setStatusBarMessage(0);
    humanPlayer->setFalseToAllEnabledCards();

    deleteCardsInCentralFromTable();

    for(int i=0; i<8;i++){
        if(cardsInCentral[i] == -1){
            cardsInCentral[i] = humanPlayer->cards[clickedCardNumber];
            break;
        }
    }
    setCardsInCentralPosition();
    addCardsInCentralToTable();

    humanPlayer->cards[clickedCardNumber] = -1;

    deleteRect(humanPlayer->cardsPosition[clickedCardNumber],cardUtil->getCardSize());

    whoWinTheRound(true,false);
}

void DesktopCore::whoWinTheRound(bool lastClickedIsPlayer, bool allowed){
    int cardsInCentralCount = cardUtil->howManyCardsInCentral(cardsInCentral);
    bool newRound = false;

    if((cardsInCentralCount+1)%2 == 0){
        if((cardUtil->getCardRow(cardsInCentral[0]) == cardUtil->getCardRow(cardsInCentral[cardsInCentralCount])) || (cardUtil->getCardRow(cardsInCentral[cardsInCentralCount]) == 0)){
            if(lastClickedIsPlayer){
                if(bot->haveBotTrumpCard(enemyPlayer->cards,cardsInCentral))
                    //QTimer::singleShot(speed,this,SLOT(botInAction()));
                    botTimer->start();
                else{
                    humanPlayer->points += cardUtil->howManyPoints(cardsInCentral);
                    isPlayerStartNextTurn = true;
                    isPlayerWonTheLastTurn = true;
                    newRound = true;
                }
            }else{
                if(cardUtil->havePlayerTrumpCard(humanPlayer->cards,cardsInCentral[0]) && !allowed){
                    cardUtil->setPlayerEnabledCardsWhenHaveTrumpCard(humanPlayer->cards,humanPlayer->getEnabledCards(),cardsInCentral[0]);
                    mainWindow->setAllowButton();
                }else{
                    enemyPlayer->points += cardUtil->howManyPoints(cardsInCentral);
                    isPlayerStartNextTurn = false;
                    isPlayerWonTheLastTurn = false;
                    newRound = true;
                }
            }
        }else{
            if(lastClickedIsPlayer){
                enemyPlayer->points += cardUtil->howManyPoints(cardsInCentral);
                isPlayerStartNextTurn = false;
                isPlayerWonTheLastTurn = false;
                newRound = true;
            }else{
                humanPlayer->points += cardUtil->howManyPoints(cardsInCentral);
                isPlayerStartNextTurn = true;
                isPlayerWonTheLastTurn = true;
                newRound = true;
            }
        }
    }else{
        if(lastClickedIsPlayer){
            mainWindow->setStatusBarMessage(DesktopCore::EnemyInAction);           
            //QTimer::singleShot(speed,this,SLOT(botInAction()));
            //inTimer = true;
            //timer->singleShot(speed,this,SLOT(botInAction()));
            botTimer->start();
        }else{
            mainWindow->setStatusBarMessage(DesktopCore::PlayerInAction);
            humanPlayer->setTrueToAllEnabledCards();
        }
    }
    if(newRound){
        //QTimer::singleShot(speed,this,SLOT(deleteCardsInCentral()));
        //inTimer = true;
        //timer->singleShot(speed,this,SLOT(deleteCardsInCentral()));
        dcicTimer->start();
    }
}

void DesktopCore::deleteCardsInCentral(){
    deleteCardsInCentralFromTable();
    cardUtil->deleteCardsInCentral(cardsInCentral);

    addCardsToEnemyAndPlayer();
    mainWindow->update();
    if((cardSum == -1) && cardUtil->isNullCardsInEnemyAndPlayer(enemyPlayer->cards,humanPlayer->cards) ){
        endOfGame();
    }else{
        if(isPlayerStartNextTurn){
            mainWindow->setStatusBarMessage(DesktopCore::PlayerInAction);
            humanPlayer->setTrueToAllEnabledCards();
            //inTimer = false;
        }else{
            mainWindow->setStatusBarMessage(DesktopCore::EnemyInAction);
            //QTimer::singleShot(speed,this,SLOT(botInAction()));
            //timer->singleShot(speed,this,SLOT(botInAction()));
            botTimer->start();
        }
    }
}

void DesktopCore::endOfGame(){
    setInGame(false);
    humanPlayer->setFalseToAllEnabledCards();
    mainWindow->setStatusBarMessage(DesktopCore::Delete);

    if(enemyPlayer->points > humanPlayer->points)
        enemyPlayer->winRounds++;
    if(enemyPlayer->points < humanPlayer->points)
        humanPlayer->winRounds++;
    if(enemyPlayer->points == humanPlayer->points){
        if(isPlayerWonTheLastTurn)
            humanPlayer->winRounds++;
        else
            enemyPlayer->winRounds++;
    }

    std::cout << "A Jatek vege " << std::endl;
    std::cout << "Az ellenfel pontszama: " << enemyPlayer->points << std::endl;
    std::cout << "A jatekos pontszama: " << humanPlayer->points << std::endl;
    mainWindow->endOfGame(enemyPlayer->points,humanPlayer->points,enemyPlayer->winRounds,humanPlayer->winRounds);
}

DesktopCore::~DesktopCore(){
    delete bot;
    delete cardUtil;
    delete enemyPlayer;
    delete humanPlayer;

    delete botTimer;
    delete dcicTimer;
}
