#ifndef DESKTOPCORE_H
#define DESKTOPCORE_H

#include <QtCore/QObject>

#include "mainwindow.h"
#include "humanplayer.h"
#include "player.h"
#include "cardutil.h"

class QSize;
class QPoint;
class Bot;

class QTimer;

class DesktopCore : public QObject{
    Q_OBJECT
    public:
        DesktopCore(MainWindow *);
        ~DesktopCore();
        
        CardUtil* cardUtil;
        Player* enemyPlayer;
        HumanPlayer* humanPlayer;

        int cardsInCentral[8]; //8, because the max card is 2*4
        QPoint cardsInCentralPosition[8];

        void startGame(int);
        int* getSortOfDeckOfCards();
        void setBot();//if the player playing in local game, the enemy is a roBot
        void setDesktopSize(QSize);
        void addDeckOfCards(); //Adds the deck of cards to the main window
        void setCardsPosition(); //Set the enemy's and player's cards positions
        void addCardsToTable(); //Add enemy,player and the central cards to the table
        void addCardsToEnemyAndPlayer(); //if true, This method is add cards to enemy and player
        void enemyClickToCard(int);
        void playerClickToCard(int);
        void whoWinTheRound(bool,bool); //This method is analyzing the cardsInCentral
        void deleteDeckOfCardsRect();
        void stopTimers();
        int getSpeed();
        void setSpeed(int mspeed);

        //inline
        inline void setCardSum(){ cardSum=31;}
        inline int getCardSum(){return cardSum;}
        //inline void setSpeed(int mspeed){ speed=mspeed;}
        //inline int getSpeed(){ return speed;}
        inline void setInGame(bool minGame){inGame=minGame;}
        inline bool getInGame(){return inGame;}
        inline void setBackCard(QString mbackCard){backCard = mbackCard;}
        inline QString getBackCard(){ return backCard;}

        //delete
        void deleteRect(QPoint,QSize);

        void addCardsInCentralToTable();
        void setCardsInCentralPosition();
        void deleteCardsInCentralFromTable();

    private:
        enum StatusBarMessage{Delete,EnemyStart,PlayerStart,EnemyInAction,PlayerInAction};
        
        Bot* bot;
        MainWindow *mainWindow;

        bool inGame;
        //bool inTimer;
        bool isPlayerStartNextTurn;
        bool isPlayerWonTheLastTurn;
        QSize windowSize;
        int cardSum; //Number of the cards in deck of cards
        //int speed;
        QString backCard;
        QTimer *botTimer;
        QTimer *dcicTimer; //delteCardsInCentral Timer
        int sortOfDeckOfCards[32]; //That which card which one

        void endOfGame(); //This method run, when the game is end

    private slots:
        void botInAction();
        void deleteCardsInCentral();


};

#endif // DESKTOPCORE_H
