#include "newgamedialog.h"
#include "desktopcore.h"
#include "mainwindow.h"
#include "resultdialog.h"
#include "aboutdialog.h"

#include <QtCore/QPoint>
#include <QtCore/QLocale>
#include <QtCore/QTranslator>
#include <QtGui/QApplication>
#include <QtGui/QWidget>
#include <QtGui/QAction>
#include <QtGui/QMenuBar>
#include <QtGui/QStatusBar>
#include <QtGui/QMenu>
#include <QtGui/QLayout>
#include <QtGui/QPainter>
#include <QtGui/QImage>
#include <QtGui/QPushButton>
#include <QtGui/QMessageBox>
#include <QtGui/QActionGroup>

#include <QTimer>

#include <iostream>

MainWindow::MainWindow(){
    resize(700,550);
    setMinimumSize(700,550);
    //setWindowTitle("QZsir");
    setWindowTitle(qApp->applicationName());
    setWindowIcon(QIcon(QString(":/pics/icon.png")));

    desktopView = new DesktopView();
    desktopCore = new DesktopCore(this);

    resizedWindow = true;

    //Menus
    initMenuBar();
    //Statusbar
    initStatusBar();

    allowButton = NULL;

    gameMode = 0;

    setMouseTracking(true);
}

void MainWindow::initMenuBar(){
    //"New" menu
    //QAction *newGame = new QAction(QString::fromUtf8("Új játék"),this);
    QAction *newGame = new QAction(tr("New game"),this);
    newGame->setShortcut(Qt::Key_F1);
    connect(newGame,SIGNAL(triggered()),this,SLOT(newGameAct()));
    //QAction *quitAction = new QAction(QString::fromUtf8("Kilépés"),this);
    QAction *quitAction = new QAction(tr("Exit"),this);
    quitAction->setShortcut(Qt::Key_Escape);
    connect(quitAction,SIGNAL(triggered()),this,SLOT(close()));

    //QMenu *gameMenu = new QMenu(QString::fromUtf8("Játék"));
    QMenu *gameMenu = new QMenu(tr("Game"));
    gameMenu->addAction(newGame);
    gameMenu->addAction(quitAction);

    //"Settings" menu
    //msec500 = new QAction(QString::fromUtf8("Fél másodperc"),this);
    msec500 = new QAction(tr("0.5 secundum"),this);
    msec500->setShortcut(Qt::Key_F2);
    //msec1000 = new QAction(QString::fromUtf8("Egy másodperc"),this);
    msec1000 = new QAction(tr("1 secundum"),this);
    msec1000->setShortcut(Qt::Key_F3);
    //msec1500 = new QAction(QString::fromUtf8("Másfél másodperc"),this);
    msec1500 = new QAction(tr("1.5 secundum"),this);
    msec1500->setShortcut(Qt::Key_F4);

    msec500->setCheckable(true);
    msec1000->setCheckable(true);
    msec1500->setCheckable(true);

    connect(msec500,SIGNAL(triggered()),this,SLOT(speedAct()));
    connect(msec1000,SIGNAL(triggered()),this,SLOT(speedAct()));
    connect(msec1500,SIGNAL(triggered()),this,SLOT(speedAct()));

    //QMenu *speedMenu = new QMenu(QString::fromUtf8("Sebesség"));
    QMenu *speedMenu = new QMenu(tr("Speed"));
    speedMenu->addAction(msec500);
    speedMenu->addAction(msec1000);
    speedMenu->addAction(msec1500);

    QActionGroup *speedActionGroup = new QActionGroup(this);
    speedActionGroup->addAction(msec500);
    speedActionGroup->addAction(msec1000);
    speedActionGroup->addAction(msec1500);

    msec1000->setChecked(true);

    //QMenu *settingsMenu = new QMenu(QString::fromUtf8("Beállítások"));
    QMenu *settingsMenu = new QMenu(tr("Settings"));
    settingsMenu->addMenu(speedMenu);

    //"Help" menu
    //QAction *aboutQt = new QAction(QString::fromUtf8("Qt névjegy"),this);
    QAction *aboutQt = new QAction(tr("About Qt"),this);
    aboutQt->setShortcut(Qt::Key_F5);
    connect(aboutQt,SIGNAL(triggered()),qApp,SLOT(aboutQt()));
    //QAction *aboutQZsirozas = new QAction(QString::fromUtf8("Névjegy"),this);
    QAction *aboutQZsir = new QAction(tr("About"),this);
    aboutQZsir->setShortcut(Qt::Key_F6);
    connect(aboutQZsir,SIGNAL(triggered()),this,SLOT(aboutAct()));

    //QMenu *helpMenu = new QMenu(QString::fromUtf8("Segítség"));
    QMenu *helpMenu = new QMenu(tr("Help"));
    helpMenu->addAction(aboutQt);
    helpMenu->addAction(aboutQZsir);

    //MenuBar
    QMenuBar *menuBar = new QMenuBar();
    menuBar->addMenu(gameMenu);
    menuBar->addMenu(settingsMenu);
    menuBar->addMenu(helpMenu);

    menuBar->setFixedHeight(22);

    //This is the menuBar Y size, becouse this remove the backgroung size
    desktopView->menuBarSize = 22;

    setMenuBar(menuBar);
}

void MainWindow::initStatusBar(){
    QStatusBar *statusBar = new QStatusBar();
    statusBar->setSizeGripEnabled(true);
    statusBar->setFixedHeight(22);
    desktopView->statusBarHeight = statusBar->size().height();
    //statusBar->showMessage(QString::fromUtf8("Üdvözöllek a QZsirozas játékban!"));
    statusBar->showMessage(tr("Welcome to QZsir!"));

    setStatusBar(statusBar);
}

void MainWindow::resizeEvent(QResizeEvent* event){
    Q_UNUSED(event);
    resizedWindow = true;
}

void MainWindow::paintEvent(QPaintEvent* event){
    Q_UNUSED(event);
    if(resizedWindow){
        desktopView->setDesktopBackground(size());
        desktopCore->setDesktopSize(size());

        desktopCore->addDeckOfCards();
        if(desktopCore->getInGame()){
            desktopCore->setDesktopSize(this->size());

            desktopCore->setCardsPosition();

            //Set Player cards rect in HumanPlayer class
            desktopCore->humanPlayer->setPlayerCardsRect(desktopCore->cardUtil->getCardSize(),desktopView->menuBarSize);

            desktopCore->addCardsToTable();

            if(allowButton)
                setAllowButtonPosition();

        }
        resizedWindow = false;
    }
    QPainter p(this);
    p.drawImage(0,0+desktopView->menuBarSize,*desktopView->getDesktopBmp());
    p.end();
}

void MainWindow::setWhoStartTheLastRound(int id){
    whoStartTheLastRound = id;
}

void MainWindow::mousePressEvent(QMouseEvent* event){
    int playerClickedCardID = desktopCore->humanPlayer->selectedCard(event->pos());
    if(playerClickedCardID >= 0){
        delete allowButton;
        allowButton = NULL;

        setCursor(Qt::ArrowCursor);

        desktopCore->playerClickToCard(playerClickedCardID);
        desktopCore->humanPlayer->selectedCardId = -1;
        update();
    }
}

void MainWindow::mouseMoveEvent(QMouseEvent* event){
    int oldSelectedCardId = desktopCore->humanPlayer->selectedCardId;
    int newSelectedCardId = desktopCore->humanPlayer->selectedCard(event->pos());
    if(oldSelectedCardId != newSelectedCardId ){
        desktopCore->humanPlayer->selectedCardId = newSelectedCardId;

        if(newSelectedCardId != -1){
            //desktopView->drawCard(desktopCore->humanPlayer->cardsPosition[newSelectedCardId],QImage(QString(":/pics/cards/"+QString::number(desktopCore->humanPlayer->cards[newSelectedCardId])+".png")),true);
            desktopView->drawCard(desktopCore->humanPlayer->cardsPosition[newSelectedCardId],desktopCore->humanPlayer->cards[newSelectedCardId],true);
            setCursor(Qt::PointingHandCursor);
            if(oldSelectedCardId != -1){
                //desktopView->drawCard(desktopCore->humanPlayer->cardsPosition[oldSelectedCardId],QImage(QString(":/pics/cards/"+QString::number(desktopCore->humanPlayer->cards[oldSelectedCardId])+".png")),false);
                desktopView->drawCard(desktopCore->humanPlayer->cardsPosition[oldSelectedCardId],desktopCore->humanPlayer->cards[oldSelectedCardId],false);
                setCursor(Qt::ArrowCursor);
            }
        }else{
            //desktopView->drawCard(desktopCore->humanPlayer->cardsPosition[oldSelectedCardId],QImage(QString(":/pics/cards/"+QString::number(desktopCore->humanPlayer->cards[oldSelectedCardId])+".png")),false);
            desktopView->drawCard(desktopCore->humanPlayer->cardsPosition[oldSelectedCardId],desktopCore->humanPlayer->cards[oldSelectedCardId],false);
            setCursor(Qt::ArrowCursor);
        }
        update();
    }
}

void MainWindow::newGameAct(){
    if(desktopCore->getInGame()){
        QMessageBox msgBox;
        //msgBox.setText(QString::fromUtf8("Biztos új játékos akarsz kezdeni?"));
        msgBox.setWindowTitle(tr("New game?"));
        msgBox.setText(tr("Do you want start new game?"));
        msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
        int ret = msgBox.exec();

        if(ret == QMessageBox::Yes){
            desktopCore->enemyPlayer->deleteCards();
            desktopCore->humanPlayer->deleteCards();
            desktopCore->enemyPlayer->deletePoints();
            desktopCore->humanPlayer->deletePoints();
            desktopCore->enemyPlayer->deleteWinRounds();
            desktopCore->humanPlayer->deleteWinRounds();

            desktopCore->deleteCardsInCentralFromTable();
            desktopCore->cardUtil->deleteCardsInCentral(desktopCore->cardsInCentral);

            if(allowButton != NULL){
                allowButton->setVisible(false);
                allowButton = NULL;
            }

            desktopCore->stopTimers();

            desktopCore->setInGame(false);
            desktopCore->humanPlayer->setFalseToAllEnabledCards();

            desktopCore->setCardSum();
            resizedWindow = true;
            update();

            newGameAct();
        }
    }else{
        NewGameDialog dial(gameMode);
        int result = dial.exec();
        if(result == 1){
            gameMode = dial.getSelectedRadioButtonId();

            desktopCore->startGame(gameMode);
            update();
        }
    }
}

void MainWindow::endOfGame(int enemyPoints,int playerPoints,int enemyWinRounds,int playerWinRounds){
    resultDialog dial(enemyPoints,playerPoints,enemyWinRounds,playerWinRounds);
    int result = dial.exec();
    if(result == 1){
        if(playerPoints>enemyPoints)
            desktopCore->startGame(0);
        if(enemyPoints>playerPoints)
            desktopCore->startGame(1);
        if(enemyPoints==playerPoints)
            desktopCore->startGame(whoStartTheLastRound);
        update();
    }else{
        desktopCore->setCardSum();
        desktopCore->deleteDeckOfCardsRect();
        desktopCore->addDeckOfCards();
        desktopCore->enemyPlayer->deleteWinRounds();
        desktopCore->humanPlayer->deleteWinRounds();
        update();
    }
}

void MainWindow::setAllowButton(){
    //allowButton = new QPushButton("Enged",this);
    allowButton = new QPushButton(tr("Allow"),this);
    connect(allowButton,SIGNAL(clicked()),this,SLOT(allowAct()));
    setAllowButtonPosition();
    allowButton->show();
}

void MainWindow::setStatusBarMessage(int id){
    switch (id){
        case 0: statusBar()->clearMessage(); break;
        //case 1: statusBar()->showMessage(QString::fromUtf8("Ellenfél kezd.")); break;
        case 1: statusBar()->showMessage(tr("Bot start the round.")); break;
        //case 2: statusBar()->showMessage(QString::fromUtf8("Játékos kezd.")); break;
        case 2: statusBar()->showMessage(tr("Player start the round.")); break;
        //case 3: statusBar()->showMessage(QString::fromUtf8("Ellenfél lép.")); break;
        case 3: statusBar()->showMessage(tr("Bot in action.")); break;
        //case 4: statusBar()->showMessage(QString::fromUtf8("Játékos lép.")); break;
        case 4: statusBar()->showMessage(tr("Player in action.")); break;
    }
}

void MainWindow::allowAct(){
    delete allowButton;
    allowButton = NULL;
    desktopCore->humanPlayer->setFalseToAllEnabledCards();
    update();
    desktopCore->whoWinTheRound(false,true);
}

void MainWindow::aboutAct(){
    AboutDialog dial;
    dial.exec();
}

void MainWindow::speedAct(){
    if(msec500->isChecked()) desktopCore->setSpeed(500);
    if(msec1000->isChecked()) desktopCore->setSpeed(1000);
    if(msec1500->isChecked()) desktopCore->setSpeed(1500);
}

void MainWindow::setAllowButtonPosition(){
    int sizeX = (int)(this->width()-allowButton->size().rwidth())/2;
    int sizeY = (int)this->height()-desktopCore->cardUtil->getCardSize().height()-50-statusBar()->size().height();
    allowButton->move(sizeX,sizeY);
}

void MainWindow::closeEvent(QCloseEvent *event){
    if(desktopCore->getInGame()){
        QMessageBox msg;
        //msg.setWindowTitle(QString::fromUtf8("Kilépsz?"));
        msg.setWindowTitle(tr("Exit?"));
        //msg.setText(QString::fromUtf8("Biztosan kiakarsz lépni?"));
        msg.setText(tr("Do you want quit?"));
        msg.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
        int ret = msg.exec();
        if(ret == QMessageBox::Yes){
            event->accept();
        }else{
            event->ignore();
        }
    }else{
        event->accept();
    }
}

MainWindow::~MainWindow(){
    delete desktopCore;
    delete desktopView;

    delete allowButton;

    delete msec500;
    delete msec1000;
    delete msec1500;
}
